@@ -1109,6 +1109,18 @@ div.cell_header {
   padding: 2px 4px 0 4px;
 }
 
+<?php
+// If we're showing the week number then put that in the top left hand corner and move
+// the day number to the centre.
+?>
+.view_week_number #month_main div.cell_header a.monthday {
+  text-align: center;
+}
+
+.view_week_number #month_main div.cell_header a.week_number {
+  position: absolute;
+}
+
 <?php // contains the list of bookings ?>
 div.booking_list {
   position: relative;
@@ -1297,7 +1309,7 @@ tbody th a:hover {
   background-color: <?php echo $row_highlight_color ?>;
 }
 
-.dwm_main.resizing tbody tr:hover t.selected,
+.dwm_main.resizing tbody tr:hover th.selected,
 .dwm_main.resizing tbody tr th.selected a:link,
 .dwm_main.resizing tbody tr th.selected a:visited {
   color: #ffffff;
@@ -1562,12 +1574,13 @@ div#div_custom_html {
     margin-bottom: 1em;
   }
 
-  .standard fieldset > div > *:not(.none) {
-    display: block;
+  .standard fieldset > div > *:not(label):not(.none) {
+    display: inline-block;
     vertical-align: middle;
   }
 
   .standard fieldset > div > label {
+    display: block;
     text-align: left;
     padding-left: 0;
     padding-right: 0;
@@ -1905,7 +1918,7 @@ form#show_my_entries input.link[type="submit"] {
     align-items: flex-start;
   }
 
-  .banner #more_info {
+  .banner .company_more_info {
     display: none;
   }
 
@@ -2568,3 +2581,17 @@ h2.date.loading::after {
   background-repeat: no-repeat;
 }
 
+div.eye {
+  height: 1em;
+}
+
+.eye svg {
+  position: relative;
+  height: 1em;
+  margin-left: -3em;
+  cursor: pointer;
+}
+
+.eye.off svg {
+  display: none;
+}
