@@ -24,31 +24,11 @@ use MRBS\Form\FieldInputTime;
 use MRBS\Form\FieldSelect;
 use MRBS\Form\FieldSpan;
 use MRBS\Form\FieldTextarea;
+use MRBS\Form\FieldTimeWithUnits;
 
 require "defaultincludes.inc";
 require_once "mrbs_sql.inc";
 
-
-// $max_unit can be set to 'seconds', 'minutes', 'hours', etc. and
-// can be used to specify the maximum unit to return.
-function get_time_unit_options($max_unit=null)
-{
-  $options = array();
-  $units = array('seconds', 'minutes', 'hours', 'days', 'weeks');
-
-  foreach ($units as $unit)
-  {
-    $options[$unit] = get_vocab($unit);
-    if (isset($max_unit) && ($max_unit == $unit))
-    {
-      break;
-    }
-  }
-
-  return $options;
-}
-
-
 function get_timezone_options()
 {
   global $zoneinfo_outlook_compatible;
@@ -59,7 +39,7 @@ function get_timezone_options()
 
   foreach ($timezone_identifiers as $value)
   {
-    if (strpos($value, '/') === FALSE)
+    if (utf8_strpos($value, '/') === FALSE)
     {
       // There are some timezone identifiers (eg 'UTC') on some operating
       // systems that don't fit the Continent/City model.   We'll put them
@@ -336,62 +316,27 @@ function get_fieldset_create_ahead()
   global $min_create_ahead_secs, $max_create_ahead_secs,
          $min_create_ahead_enabled, $max_create_ahead_enabled;
 
-  $min_create_ahead_value = $min_create_ahead_secs;
-  toTimeString($min_create_ahead_value, $min_create_ahead_units);
-  $max_create_ahead_value = $max_create_ahead_secs;
-  toTimeString($max_create_ahead_value, $max_create_ahead_units);
-
-  $options = get_time_unit_options();
-
   $fieldset = new ElementFieldset();
   $fieldset->addLegend(get_vocab('booking_creation'));
 
-
   // Minimum create ahead
-  $field = new FieldDiv();
-
-  $checkbox = new ElementInputCheckbox();
-  $checkbox->setAttributes(array('name'  => 'area_min_create_ahead_enabled',
-                                 'class' => 'enabler'))
-           ->setChecked($min_create_ahead_enabled);
-
-  $input = new ElementInputNumber();
-  $input->setAttributes(array('name'  => 'area_min_create_ahead_value',
-                              'value' => $min_create_ahead_value));
-
-  $select = new ElementSelect();
-  $select->setAttribute('name', 'area_min_create_ahead_units')
-         ->addSelectOptions($options, array_search($min_create_ahead_units, $options), true);
-
-  $field->setLabel(get_vocab('min_book_ahead'))
-        ->addControlElement($checkbox)
-        ->addControlElement($input)
-        ->addControlElement($select);
-
+  $param_names = array(
+      'enabler'  => 'area_min_create_ahead_enabled',
+      'quantity' => 'area_min_create_ahead_value',
+      'units'    => 'area_min_create_ahead_units',
+    );
+  $field = new FieldTimeWithUnits($param_names, $min_create_ahead_enabled, $min_create_ahead_secs);
+  $field->setLabel(get_vocab('min_book_ahead'));
   $fieldset->addElement($field);
 
-
   // Maximum create ahead
-  $field = new FieldDiv();
-
-  $checkbox = new ElementInputCheckbox();
-  $checkbox->setAttributes(array('name'  => 'area_max_create_ahead_enabled',
-                                 'class' => 'enabler'))
-           ->setChecked($max_create_ahead_enabled);
-
-  $input = new ElementInputNumber();
-  $input->setAttributes(array('name'  => 'area_max_create_ahead_value',
-                              'value' => $max_create_ahead_value));
-
-  $select = new ElementSelect();
-  $select->setAttribute('name', 'area_max_create_ahead_units')
-         ->addSelectOptions($options, array_search($max_create_ahead_units, $options), true);
-
-  $field->setLabel(get_vocab('max_book_ahead'))
-        ->addControlElement($checkbox)
-        ->addControlElement($input)
-        ->addControlElement($select);
-
+  $param_names = array(
+      'enabler'  => 'area_max_create_ahead_enabled',
+      'quantity' => 'area_max_create_ahead_value',
+      'units'    => 'area_max_create_ahead_units',
+    );
+  $field = new FieldTimeWithUnits($param_names, $max_create_ahead_enabled, $max_create_ahead_secs);
+  $field->setLabel(get_vocab('max_book_ahead'));
   $fieldset->addElement($field);
 
   return $fieldset;
@@ -403,61 +348,27 @@ function get_fieldset_delete_ahead()
   global $min_delete_ahead_secs, $max_delete_ahead_secs,
          $min_delete_ahead_enabled, $max_delete_ahead_enabled;
 
-  $min_delete_ahead_value = $min_delete_ahead_secs;
-  toTimeString($min_delete_ahead_value, $min_delete_ahead_units);
-  $max_delete_ahead_value = $max_delete_ahead_secs;
-  toTimeString($max_delete_ahead_value, $max_delete_ahead_units);
-
-  $options = get_time_unit_options();
-
   $fieldset = new ElementFieldset();
   $fieldset->addLegend(get_vocab('booking_deletion'));
 
   // Minimum delete ahead
-  $field = new FieldDiv();
-
-  $checkbox = new ElementInputCheckbox();
-  $checkbox->setAttributes(array('name'  => 'area_min_delete_ahead_enabled',
-                                 'class' => 'enabler'))
-           ->setChecked($min_delete_ahead_enabled);
-
-  $input = new ElementInputNumber();
-  $input->setAttributes(array('name'  => 'area_min_delete_ahead_value',
-                              'value' => $min_delete_ahead_value));
-
-  $select = new ElementSelect();
-  $select->setAttribute('name', 'area_min_delete_ahead_units')
-         ->addSelectOptions($options, array_search($min_delete_ahead_units, $options), true);
-
-  $field->setLabel(get_vocab('min_book_ahead'))
-        ->addControlElement($checkbox)
-        ->addControlElement($input)
-        ->addControlElement($select);
-
+  $param_names = array(
+      'enabler'  => 'area_min_delete_ahead_enabled',
+      'quantity' => 'area_min_delete_ahead_value',
+      'units'    => 'area_min_delete_ahead_units',
+    );
+  $field = new FieldTimeWithUnits($param_names, $min_delete_ahead_enabled, $min_delete_ahead_secs);
+  $field->setLabel(get_vocab('min_book_ahead'));
   $fieldset->addElement($field);
 
-
   // Maximum delete ahead
-  $field = new FieldDiv();
-
-  $checkbox = new ElementInputCheckbox();
-  $checkbox->setAttributes(array('name'  => 'area_max_delete_ahead_enabled',
-                                 'class' => 'enabler'))
-           ->setChecked($max_delete_ahead_enabled);
-
-  $input = new ElementInputNumber();
-  $input->setAttributes(array('name'  => 'area_max_delete_ahead_value',
-                              'value' => $max_delete_ahead_value));
-
-  $select = new ElementSelect();
-  $select->setAttribute('name', 'area_max_delete_ahead_units')
-         ->addSelectOptions($options, array_search($max_delete_ahead_units, $options), true);
-
-  $field->setLabel(get_vocab('max_book_ahead'))
-        ->addControlElement($checkbox)
-        ->addControlElement($input)
-        ->addControlElement($select);
-
+  $param_names = array(
+      'enabler'  => 'area_max_delete_ahead_enabled',
+      'quantity' => 'area_max_delete_ahead_value',
+      'units'    => 'area_max_delete_ahead_units',
+    );
+  $field = new FieldTimeWithUnits($param_names, $max_delete_ahead_enabled, $max_delete_ahead_secs);
+  $field->setLabel(get_vocab('max_book_ahead'));
   $fieldset->addElement($field);
 
   return $fieldset;
@@ -579,7 +490,7 @@ function get_fieldset_max_secs()
 
     $max = $max_secs_per_interval_area[$interval_type];
     toTimeString($max, $units, true, $max_unit);
-    $options = get_time_unit_options($max_unit);
+    $options = Form::getTimeUnitOptions($max_unit);
 
     $select = new ElementSelect();
     $select->setAttribute('name', "area_max_secs_per_${interval_type}_units")
@@ -655,7 +566,7 @@ function get_fieldset_max_duration()
   // Times
   $max_duration_value = $max_duration_secs;
   toTimeString($max_duration_value, $max_duration_units);
-  $options = get_time_unit_options();
+  $options = Form::getTimeUnitOptions();
 
   $select = new ElementSelect();
   $select->setAttribute('name', 'area_max_duration_units')
