@@ -1,6 +1,7 @@
 <?php
 namespace MRBS;
 
+use MRBS\Form\FieldTimeWithUnits;
 use MRBS\Form\Form;
 use MRBS\Form\ElementDiv;
 use MRBS\Form\ElementFieldset;
@@ -912,7 +913,10 @@ function get_field_skip_conflicts($disabled=false)
 function get_fieldset_registration()
 {
   global $enable_registration;
-  global $allow_registration, $enable_registrant_limit, $registrant_limit;
+  global $allow_registration, $registrant_limit_enabled, $registrant_limit;
+  global $registration_opens, $registration_opens_enabled;
+  global $registration_closes, $registration_closes_enabled;
+  global $enable_periods, $periods_booking_opens;
 
   if (!$enable_registration || !is_book_admin())
   {
@@ -925,16 +929,17 @@ function get_fieldset_registration()
 
   $field = new FieldInputCheckbox();
   $field->setLabel(get_vocab('allow_registration'))
-    ->setControlAttribute('name', 'allow_registration')
-    ->setChecked($allow_registration);
-
+        ->setControlAttributes(array(
+            'name' => 'allow_registration'))
+        ->setChecked($allow_registration);
   $fieldset->addElement($field);
 
   $field = new FieldInputCheckbox();
-  $field->setLabel(get_vocab('enable_registrant_limit'))
-    ->setControlAttribute('name', 'enable_registrant_limit')
-    ->setChecked($enable_registrant_limit);
-
+  $field->setLabel(get_vocab('registrant_limit_enabled'))
+        ->setControlAttributes(array(
+            'name' => 'registrant_limit_enabled',
+            'class' => 'enabler'))
+        ->setChecked($registrant_limit_enabled);
   $fieldset->addElement($field);
 
   $field = new FieldInputNumber();
@@ -946,7 +951,51 @@ function get_fieldset_registration()
               'value'    => $registrant_limit
             )
           );
+  $fieldset->addElement($field);
 
+  // Registration opens and closes
+  // The suffix text for registration opens and closes depends on whether
+  // periods are being used.
+  if ($enable_periods)
+  {
+    $time = strtotime($periods_booking_opens);
+    $time = strftime(hour_min_format(), $time);
+    $in_advance_vocab = get_vocab('in_advance_periods', $time);
+  }
+  else
+  {
+    $in_advance_vocab = get_vocab('in_advance');
+  }
+
+  // Registration opens
+  $param_names = array(
+      'enabler'  => 'registration_opens_enabled',
+      'quantity' => 'registration_opens_value',
+      'units'    => 'registration_opens_units',
+    );
+  $field = new FieldTimeWithUnits(
+      $param_names,
+      $registration_opens_enabled,
+      $registration_opens,
+      get_vocab($in_advance_vocab),
+      array('min' => 0)  // no point in opening registration after the event starts
+    );
+  $field->setLabel(get_vocab('registration_opens'));
+  $fieldset->addElement($field);
+
+  // Registration closes
+  $param_names = array(
+    'enabler'  => 'registration_closes_enabled',
+    'quantity' => 'registration_closes_value',
+    'units'    => 'registration_closes_units',
+  );
+  $field = new FieldTimeWithUnits(
+      $param_names,
+      $registration_closes_enabled,
+      $registration_closes,
+      get_vocab($in_advance_vocab)
+    );
+  $field->setLabel(get_vocab('registration_closes'));
   $fieldset->addElement($field);
 
   return $fieldset;
@@ -1173,7 +1222,7 @@ if (isset($id))
   get_area_settings($area);
 
   $private = $entry['private'];
-  if ($private_mandatory)
+  if ($private_mandatory && !is_book_admin($entry['room_id']))
   {
     $private = $private_default;
   }
@@ -1236,8 +1285,12 @@ if (isset($id))
       case 'entry_type':
       case 'tentative':
       case 'allow_registration':
-      case 'enable_registrant_limit':
       case 'registrant_limit':
+      case 'registrant_limit_enabled':
+      case 'registration_opens':
+      case 'registration_opens_enabled':
+      case 'registration_closes':
+      case 'registration_closes_enabled':
         $$column = $entry[$column];
         break;
 
@@ -1357,16 +1410,27 @@ else
 {
   // It is a new booking. The data comes from whichever button the user clicked
   $edit_type     = "series";
-  $name          = $default_name;
+  if ($default_name_display_name)
+  {
+    $name = (isset($mrbs_user)) ? $mrbs_user->display_name : '';
+  }
+  else
+  {
+    $name = $default_name;
+  }
   $create_by     = $mrbs_username;
   $description   = $default_description;
   $type          = (empty($is_mandatory_field['entry.type'])) ? $default_type : '';
   $room_id       = $room;
   $private       = $private_default;
   $tentative     = !$confirmed_default;
-  $allow_registration      = false;
-  $enable_registrant_limit = true;
-  $registrant_limit        = 1;
+  $allow_registration           = (bool) $allow_registration_default;
+  $registrant_limit             = (int) $registrant_limit_default;
+  $registrant_limit_enabled     = (bool) $registrant_limit_enabled_default;
+  $registration_opens           = (int) $registration_opens_default;
+  $registration_opens_enabled   = (bool) $registration_opens_enabled_default;
+  $registration_closes          = (int) $registration_closes_default;
+  $registration_closes_enabled  = (bool) $registration_closes_enabled_default;
 
   // Get the hour and minute, converting a period to its MRBS time
   // Set some sensible defaults
@@ -1480,21 +1544,6 @@ list($month_relative_ord, $month_relative_day) = byday_split($month_relative);
 $start_hour  = strftime('%H', $start_time);
 $start_min   = strftime('%M', $start_time);
 
-// These next 4 if statements handle the situation where
-// this page has been accessed directly and no arguments have
-// been passed to it.
-// If we have not been provided with a room_id
-if (empty( $room_id ) )
-{
-  $sql = "SELECT id
-            FROM " . _tbl('room') . "
-           WHERE disabled=0
-           LIMIT 1";
-  $res = db()->query($sql);
-  $row = $res->next_row_keyed();
-  $room_id = $row['id'];
-}
-
 // Determine the area id of the room in question first
 $area_id = mrbsGetRoomArea($room_id);
 
@@ -1605,6 +1654,16 @@ while (false !== ($row = $res->next_row_keyed()))
   $areas[$row['id']] = $row;
 }
 
+// Check that the area for this room actually exists.  This will happen if the room id in the query
+// string is invalid, usually as a result of using an out of date bookmark.
+if (!isset($areas[$area_id]))
+{
+  echo "<h1>" . get_vocab('error') . "</h1>\n";
+  echo "<p>" . get_vocab('edit_entry_nonexistent_room') . "</p>\n";
+  echo '<p><a href="' . multisite('index.php') . '">' . get_vocab('home') . "</a></p>\n";
+  // Print footer and exit
+  print_footer(true);
+}
 
 if (isset($id) && !isset($copy))
 {
