@@ -73,42 +73,48 @@ $mrbs_username = (isset($mrbs_user)) ? $mrbs_user->username : null;
 
 // Get non-standard form variables
 $form_vars = array(
-  'create_by'               => 'string',
-  'name'                    => 'string',
-  'description'             => 'string',
-  'start_seconds'           => 'int',
-  'start_date'              => 'string',
-  'end_seconds'             => 'int',
-  'end_date'                => 'string',
-  'all_day'                 => 'string',  // bool, actually
-  'type'                    => 'string',
-  'rooms'                   => 'array',
-  'original_room_id'        => 'int',
-  'ical_uid'                => 'string',
-  'ical_sequence'           => 'int',
-  'ical_recur_id'           => 'string',
-  'allow_registration'      => 'string',  // bool, actually
-  'enable_registrant_limit' => 'string',  // bool, actually
-  'registrant_limit'        => 'int',
-  'returl'                  => 'string',
-  'id'                      => 'int',
-  'rep_id'                  => 'int',
-  'edit_type'               => 'string',
-  'rep_type'                => 'int',
-  'rep_end_date'            => 'string',
-  'rep_day'                 => 'array',   // array of bools
-  'rep_interval'            => 'int',
-  'month_type'              => 'int',
-  'month_absolute'          => 'int',
-  'month_relative_ord'      => 'string',
-  'month_relative_day'      => 'string',
-  'skip'                    => 'string',  // bool, actually
-  'no_mail'                 => 'string',  // bool, actually
-  'private'                 => 'string',  // bool, actually
-  'confirmed'               => 'string',
-  'back_button'             => 'string',
-  'timetohighlight'         => 'int',
-  'commit'                  => 'string'
+  'create_by'                   => 'string',
+  'name'                        => 'string',
+  'description'                 => 'string',
+  'start_seconds'               => 'int',
+  'start_date'                  => 'string',
+  'end_seconds'                 => 'int',
+  'end_date'                    => 'string',
+  'all_day'                     => 'string',  // bool, actually
+  'type'                        => 'string',
+  'rooms'                       => 'array',
+  'original_room_id'            => 'int',
+  'ical_uid'                    => 'string',
+  'ical_sequence'               => 'int',
+  'ical_recur_id'               => 'string',
+  'allow_registration'          => 'string',  // bool, actually
+  'registrant_limit'            => 'int',
+  'registrant_limit_enabled'    => 'string',  // bool, actually
+  'registration_opens_value'    => 'int',
+  'registration_opens_units'    => 'string',
+  'registration_opens_enabled'  => 'string',  // bool, actually
+  'registration_closes_value'   => 'int',
+  'registration_closes_units'   => 'string',
+  'registration_closes_enabled' => 'string',  // bool, actually
+  'returl'                      => 'string',
+  'id'                          => 'int',
+  'rep_id'                      => 'int',
+  'edit_type'                   => 'string',
+  'rep_type'                    => 'int',
+  'rep_end_date'                => 'string',
+  'rep_day'                     => 'array',   // array of bools
+  'rep_interval'                => 'int',
+  'month_type'                  => 'int',
+  'month_absolute'              => 'int',
+  'month_relative_ord'          => 'string',
+  'month_relative_day'          => 'string',
+  'skip'                        => 'string',  // bool, actually
+  'no_mail'                     => 'string',  // bool, actually
+  'private'                     => 'string',  // bool, actually
+  'confirmed'                   => 'string',
+  'back_button'                 => 'string',
+  'timetohighlight'             => 'int',
+  'commit'                      => 'string'
 );
 
 foreach($form_vars as $var => $var_type)
@@ -121,9 +127,26 @@ foreach($form_vars as $var => $var_type)
     $$var = trim($$var);
     $$var = truncate($$var, "entry.$var");
   }
-
 }
 
+// Convert the registration opens and closes times into seconds
+if (isset($registration_opens_value) && isset($registration_opens_units))
+{
+  fromTimeString($registration_opens_value, $registration_opens_units);
+  $registration_opens = constrain_int($registration_opens_value, 4);
+}
+
+if (isset($registration_closes_value) && isset($registration_closes_units))
+{
+  fromTimeString($registration_closes_value, $registration_closes_units);
+  $registration_closes = constrain_int($registration_closes_value, 4);
+}
+
+// Convert the booleans (the custom field booleans are done later)
+$registrant_limit_enabled = ($registrant_limit_enabled) ? 1 : 0;
+$registration_opens_enabled = ($registration_opens_enabled) ? 1 : 0;
+$registration_closes_enabled = ($registration_closes_enabled) ? 1 : 0;
+
 // If they're not an admin and multi-day bookings are not allowed, then
 // set the end date to the start date
 if (!is_book_admin($rooms) && $auth['only_admin_can_book_multiday'])
@@ -208,12 +231,20 @@ foreach($fields as $field)
 
 // Form validation checks.   Normally checked for client side.
 
-// The id must be either an integer or NULL, so that subsequent code that tests whether
-// isset($id) works.  (I suppose one could use !empty instead, but there's always the
-// possibility that sites have allowed 0 in their auto-increment/serial columns.)
-if (isset($id) && ($id == ''))
+if (isset($id))
 {
-  unset($id);
+  $old_booking = get_booking_info($id, false);
+
+  // Get the old values of registration_opens, registration_closes and registrant_limit
+  // if they're not set, otherwise writing null to the database will cause the column
+  // defaults to be used.
+  foreach (['registration_opens', 'registration_closes', 'registrant_limit'] as $var)
+  {
+    if (!isset($$var))
+    {
+      $$var = $old_booking[$var];
+    }
+  }
 }
 
 // Validate the create_by variable, checking that it's the current user, unless the
@@ -303,7 +334,7 @@ if (isset($month_relative_ord) && isset($month_relative_day))
 
 // Handle private booking
 // Enforce config file settings if needed
-if ($private_mandatory)
+if ($private_mandatory && !is_book_admin())
 {
   $isprivate = $private_default;
 }
@@ -334,13 +365,12 @@ if ($no_mail)
 
 // If this is an Ajax request and we're being asked to commit the booking, then
 // we'll only have been supplied with parameters that need to be changed.  Fill in
-// the rest from the existing boking information.
+// the rest from the existing booking information.
 // Note: we assume that
 // (1) this is not a series (we can't cope with them yet)
 // (2) we always get passed start_seconds and end_seconds in the Ajax data
 if ($is_ajax && $commit)
 {
-  $old_booking = get_booking_info($id, false);
   foreach ($form_vars as $var => $var_type)
   {
     if (!isset($$var) || (($var_type == 'array') && empty($$var)))
@@ -704,8 +734,13 @@ foreach ($rooms as $room_id)
   $booking['ical_sequence'] = $ical_sequence;
   $booking['ical_recur_id'] = $ical_recur_id;
   $booking['allow_registration'] = $allow_registration;
-  $booking['enable_registrant_limit'] = $enable_registrant_limit;
   $booking['registrant_limit'] = $registrant_limit;
+  $booking['registrant_limit_enabled'] = $registrant_limit_enabled;
+  $booking['registration_opens'] = (isset($registration_opens)) ? $registration_opens : null;
+  $booking['registration_opens_enabled'] = $registration_opens_enabled;
+  $booking['registration_closes'] = (isset($registration_closes)) ? $registration_closes : null;
+  $booking['registration_closes_enabled'] = $registration_closes_enabled;
+
 
   if ($booking['rep_type'] == REP_MONTHLY)
   {
