@@ -43,6 +43,12 @@ use MRBS\Form\FieldSelect;
 
 require "defaultincludes.inc";
 
+if ($auth['type'] != 'db')
+{
+  // You shouldn't be here unless you're using 'db' authentication
+  location_header('index.php');
+}
+
 // Get non-standard form variables
 $action = get_form_var('action', 'string');
 $id = get_form_var('id', 'int');
@@ -529,6 +535,12 @@ function get_fieldset_submit_buttons($delete=false, $disabled=false, $last_admin
                                'disabled'       => $disabled,
                                'formnovalidate' => true));
 
+  if ($delete)
+  {
+    $message = get_vocab("confirm_delete_user");
+    $button->setAttribute('onclick', "return confirm('" . escape_js($message) . "');");
+  }
+
   $field->setAttribute('class', 'submit_buttons')
         ->setLabelAttribute('class', 'no_suffix')
         ->addLabelElement($button)
@@ -1221,8 +1233,8 @@ if ($initial_user_creation != 1)   // don't print the user table if there are no
     echo "<tr>";
 
     // First two columns which are the name and display name
-    echo '<th><span class="normal" data-type="title-string">' . get_vocab("users.display_name") . "</th>\n";
-    echo '<th><span class="normal" data-type="title-string">' . get_vocab("users.name") . "</th>\n";
+    echo '<th><span class="normal" data-type="title-string">' . get_vocab("users.display_name") . "</span></th>\n";
+    echo '<th><span class="normal" data-type="title-string">' . get_vocab("users.name") . "</span></th>\n";
 
     // Other column headers
     foreach ($fields as $field)
