@@ -37,7 +37,7 @@ function get_compression_wrappers()
     foreach ($wrappers as $wrapper)
     {
       if ((($wrapper == 'zip') && class_exists('ZipArchive')) ||
-          (strpos($wrapper, 'compress.') === 0))
+          (utf8_strpos($wrapper, 'compress.') === 0))
       {
         $result[] = $wrapper;
       }
@@ -56,7 +56,7 @@ function get_room_id($location, &$error)
 
   // If there's no delimiter we assume we've just been given a room name (that will
   // have to be unique).   Otherwise we split the location into its area and room parts
-  if (strpos($location, $area_room_delimiter) === FALSE)
+  if (utf8_strpos($location, $area_room_delimiter) === false)
   {
     $location_area = '';
     $location_room = $location;
@@ -255,6 +255,8 @@ function process_event($vevent)
   // Set up the booking with some defaults
   $booking = array();
   $booking['awaiting_approval'] = false;
+  $booking['private'] = false;
+  $booking['tentative'] = false;
   $booking['rep_type'] = REP_NONE;
   $booking['type'] = $import_default_type;
   $booking['room_id'] = $import_default_room;
@@ -562,7 +564,7 @@ function get_file_details_zip($file)
       $constants = $reflection->getConstants();
       foreach ($constants as $key => $value)
       {
-        if (($result === $value) && (strpos($key, 'ER_') === 0))
+        if (($result === $value) && (utf8_strpos($key, 'ER_') === 0))
         {
           $error_code = $key;
           break;
