@@ -169,19 +169,6 @@ if (false !== ($key = array_search('rooms', $edit_entry_field_order)))
   trigger_error($message, E_USER_NOTICE);
 }
 
-// Variables in a changed format in versions of MRBS > 1.7.1
-if (!empty($override_locale))
-{
-  $new_override_locale = System::getBCPlocale($override_locale);
-  if (strtolower($override_locale) != strtolower($new_override_locale))
-  {
-    $message = 'The config variable $override_locale should now be in BCP 47 format. ' .
-               "Please change '$override_locale' to '$new_override_locale'.";
-    trigger_error($message, E_USER_NOTICE);
-    $override_locale = $new_override_locale;
-  }
-}
-
 
 /********************************************************
  * Checking
@@ -369,53 +356,61 @@ define('VAR_PREFIX', 'f_');  // must begin with a letter;
 // These are the standard fields in the database tables.   If you add more
 // standard (not user defined, custom) fields, then you need to change these
 
-$standard_fields['entry'] = array('id',
-                                  'start_time',
-                                  'end_time',
-                                  'entry_type',
-                                  'repeat_id',
-                                  'room_id',
-                                  'timestamp',
-                                  'create_by',
-                                  'modified_by',
-                                  'name',
-                                  'type',
-                                  'description',
-                                  'status',
-                                  'reminded',
-                                  'info_time',
-                                  'info_user',
-                                  'info_text',
-                                  'ical_uid',
-                                  'ical_sequence',
-                                  'ical_recur_id',
-                                  'allow_registration',
-                                  'enable_registrant_limit',
-                                  'registrant_limit');
+$standard_fields['entry'] = array(
+    'id',
+    'start_time',
+    'end_time',
+    'entry_type',
+    'repeat_id',
+    'room_id',
+    'timestamp',
+    'create_by',
+    'modified_by',
+    'name',
+    'type',
+    'description',
+    'status',
+    'reminded',
+    'info_time',
+    'info_user',
+    'info_text',
+    'ical_uid',
+    'ical_sequence',
+    'ical_recur_id',
+    'allow_registration',
+    'registrant_limit',
+    'registrant_limit_enabled',
+    'registration_opens',
+    'registration_opens_enabled',
+    'registration_closes',
+    'registration_closes_enabled'
+  );
 
-$standard_fields['repeat'] = array('id',
-                                   'start_time',
-                                   'end_time',
-                                   'rep_type',
-                                   'end_date',
-                                   'rep_opt',
-                                   'room_id',
-                                   'timestamp',
-                                   'create_by',
-                                   'modified_by',
-                                   'name',
-                                   'type',
-                                   'description',
-                                   'rep_interval',
-                                   'month_absolute',
-                                   'month_relative',
-                                   'status',
-                                   'reminded',
-                                   'info_time',
-                                   'info_user',
-                                   'info_text',
-                                   'ical_uid',
-                                   'ical_sequence');
+$standard_fields['repeat'] = array(
+    'id',
+    'start_time',
+    'end_time',
+    'rep_type',
+    'end_date',
+    'rep_opt',
+    'room_id',
+    'timestamp',
+    'create_by',
+    'modified_by',
+    'name',
+    'type',
+    'description',
+    'rep_interval',
+    'month_absolute',
+    'month_relative',
+    'status',
+    'reminded',
+    'info_time',
+    'info_user',
+    'info_text',
+    'ical_uid',
+    'ical_sequence'
+  );
 
 $standard_fields['room'] = array('id',
                                  'disabled',
@@ -498,7 +493,7 @@ $area_defaults_keys = array('timezone',
                             'confirmation_enabled',
                             'confirmed_default',
                             'times_along_top',
-                            'default_area');
+                            'default_type');
 
 $area_defaults = array();
 
