@@ -82,16 +82,16 @@ var changeRepIntervalUnits = function changeRepIntervalUnits() {
     switch (repType)
     {
       case <?php echo REP_DAILY ?>:
-        text = (repInterval == 1) ? '<?php echo get_vocab('day') ?>' : '<?php echo get_vocab('days') ?>';
+        text = (repInterval === 1) ? '<?php echo get_vocab('day') ?>' : '<?php echo get_vocab('days') ?>';
         break;
       case <?php echo REP_WEEKLY ?>:
-        text = (repInterval == 1) ? '<?php echo get_vocab('week') ?>' : '<?php echo get_vocab('weeks') ?>';
+        text = (repInterval === 1) ? '<?php echo get_vocab('week') ?>' : '<?php echo get_vocab('weeks') ?>';
         break;
       case <?php echo REP_MONTHLY ?>:
-        text = (repInterval == 1) ? '<?php echo get_vocab('month') ?>' : '<?php echo get_vocab('months') ?>';
+        text = (repInterval === 1) ? '<?php echo get_vocab('month') ?>' : '<?php echo get_vocab('months') ?>';
         break;
       case <?php echo REP_YEARLY ?>:
-        text = (repInterval == 1) ? '<?php echo get_vocab('year_lc') ?>' : '<?php echo get_vocab('years') ?>';
+        text = (repInterval === 1) ? '<?php echo get_vocab('year_lc') ?>' : '<?php echo get_vocab('years') ?>';
         break;
       default:
         text = units.text();
@@ -461,19 +461,31 @@ function validate(form)
     return false;
   }
 
-  <?php
-  // Check that there's a sensible value for rep_interval.   Only necessary
-  // if the browser doesn't support the HTML5 min and step attributes
-  ?>
-  if (!("min" in testInput) || !(("step" in testInput)))
+  <?php // Repeat checks ?>
+  var repType = form.find('input:radio[name=rep_type]:checked').val();
+  if ((repType !== undefined) && (parseInt(repType, 10) !== <?php echo REP_NONE ?>))
   {
-    var repType = form.find('input:radio[name=rep_type]:checked').val();
-
-    if ((repType !== <?php echo REP_NONE ?>) && (form.find('#rep_interval').val() < 1))
+    <?php
+    // Check that there's a sensible value for rep_interval.   Only necessary
+    // if the browser doesn't support the HTML5 min and step attributes
+    ?>
+    if ((!("min" in testInput) || !(("step" in testInput))) &&
+        (form.find('#rep_interval').val() < 1))
     {
       window.alert("<?php echo escape_js(get_vocab('invalid_rep_interval')) ?>");
       return false;
     }
+    <?php
+    // Check that the repeat end date has been set (people often forget to do so).  If it's the
+    // same as the entry end date then it probably hasn't.
+    ?>
+    if ($('input[name="rep_end_date"]').val() === $('input[name="end_date"]').val())
+    {
+      if (!window.confirm("<?php echo escape_js(get_vocab('confirm_rep_end_date')) ?>"))
+      {
+        return false;
+      }
+    }
   }
 
   <?php
@@ -1289,18 +1301,18 @@ $(document).on('page_ready', function() {
   });
 
   form.on('submit', function() {
+      var result = true;
       if ($(this).data('submit') === 'save_button')
       {
         <?php // Only validate the form if the Save button was pressed ?>
-        var result = validate($(this));
+        result = validate($(this));
         if (!result)
         {
           <?php // Clear the data flag if the validation failed ?>
           $(this).removeData('submit');
         }
-        return result;
       }
-      return true;
+      return result;
     });
 
   <?php
@@ -1488,4 +1500,32 @@ $(document).on('page_ready', function() {
     nameInput.trigger('focus');
   }
 
+
+  <?php
+  // If the allow_registration checkbox is changed then we need to toggle the
+  // disabled property of all the controls in the registration fieldset.  However,
+  // because some of them are also enabler groups themselves then if allow_registration
+  // is checked we need to trigger a change event on them so that they set their own
+  // disabled properties appropriately.
+  ?>
+  var allowRegistration = $('#allow_registration');
+  allowRegistration.on('change', function() {
+      var registration = $('#registration');
+      var allowRegistrationChecked = $(this).is(':checked');
+      registration.find('input, select').not($(this)).prop('disabled', !allowRegistrationChecked);
+      if (allowRegistrationChecked)
+      {
+        registration.find('.enabler').trigger('change');
+      }
+    })
+    .trigger('change');
+
+  <?php
+  // Enable the checkboxes which may have been disabled, otherwise their values
+  // will not be posted.
+  ?>
+  form.submit(function() {
+      $('#registration').find('input[type="checkbox"]').prop('disabled', false);
+    });
+
 });
