@@ -74,21 +74,14 @@ function rectanglesIdentical(r1, r2)
 function rectanglesOverlap(r1, r2)
 {
   <?php
-  // If any of the sides of r1 is clear of the opposite side of r2, then the two
-  // rectangles don't overlap.   Otherwise they must do.   We allow the sides to
-  // be on top of each other (ie we use >= and <=).
+  // If none of the sides of r1 is clear of the opposite side of r2, then the two
+  // rectangles overlap. We allow the sides to be on top of each other (ie we use
+  // < and > instead of <= and >=).
   ?>
-  if ((Math.round(r1.n - r2.s) >= 0) ||
-      (Math.round(r1.s - r2.n) <= 0) ||
-      (Math.round(r1.w - r2.e) >= 0) ||
-      (Math.round(r1.e - r2.w) <= 0))
-  {
-    return false;
-  }
-  else
-  {
-    return true;
-  }
+  return ((Math.round(r1.n - r2.s) < 0) &&
+          (Math.round(r1.s - r2.n) > 0) &&
+          (Math.round(r1.w - r2.e) < 0) &&
+          (Math.round(r1.e - r2.w) > 0));
 }
 
 
@@ -137,7 +130,7 @@ function getDataName(jqObject)
 
 
 var Table = {
-  selector: ".dwm_main:not('#month_main')",
+  selector: ".dwm_main",
   borderLeftWidth: undefined,
   borderTopWidth: undefined,
   bookedMap: [],
@@ -387,7 +380,7 @@ var Table = {
 
   size: function() {
       <?php // Don't do anything if this is the all-rooms week view ?>
-      if ((args.view == 'week') && args.view_all)
+      if ((args.view === 'week') && args.view_all)
       {
         return;
       }
@@ -660,8 +653,13 @@ $(document).on('page_ready', function() {
   $(Table.selector).on('tableload', function() {
       var table = $(this);
 
-      <?php // Don't do anything if this is an empty table or the all-rooms week view ?>
-      if (((args.view == 'week') && args.view_all) ||
+      <?php
+      // Don't do anything if this is an empty table, or the all-rooms week view,
+      // or the month view
+      ?>
+
+      if ((args.view === 'month') ||
+          ((args.view === 'week') && args.view_all) ||
           table.find('tbody').data('empty'))
       {
         return;
@@ -702,8 +700,8 @@ $(document).on('page_ready', function() {
             // the week view) or select multiple rooms (in the day view) then
             // constrain the box to fit in the current slot width/height
             ?>
-            if (((args.view == 'week') && <?php echo ($auth['only_admin_can_book_repeat']) ? 'true' : 'false'?>) ||
-                ((args.view == 'day') && <?php echo ($auth['only_admin_can_select_multiroom']) ? 'true' : 'false'?>))
+            if (((args.view === 'week') && <?php echo ($auth['only_admin_can_book_repeat']) ? 'true' : 'false'?>) ||
+                ((args.view === 'day') && <?php echo ($auth['only_admin_can_select_multiroom']) ? 'true' : 'false'?>))
             {
               <?php
               if ($times_along_top)
@@ -881,7 +879,6 @@ $(document).on('page_ready', function() {
             queryString += '&site=' + encodeURIComponent(args.site);
           }
           window.location = 'edit_entry.php?' + queryString;
-          return;
         };
 
 
@@ -1281,8 +1278,8 @@ $(document).on('page_ready', function() {
             }
             if (!args.isBookAdmin)
             {
-              if (((args.view == 'week') && <?php echo ($auth['only_admin_can_book_repeat']) ? 'true' : 'false'?>) ||
-                  ((args.view == 'day') && <?php echo ($auth['only_admin_can_select_multiroom']) ? 'true' : 'false'?>))
+              if (((args.view === 'week') && <?php echo ($auth['only_admin_can_book_repeat']) ? 'true' : 'false'?>) ||
+                  ((args.view === 'day') && <?php echo ($auth['only_admin_can_select_multiroom']) ? 'true' : 'false'?>))
               {
                 <?php
                 // If we're in the week view then if non-admins aren't allowed to
