@@ -83,8 +83,13 @@ class AuthLdap extends Auth
   const LDAP_CLIENT_LOOP =                    0x60;
   const LDAP_REFERRAL_LIMIT_EXCEEDED =        0x61;
 
+  // Default ports
+  const DEFAULT_PORT_LDAP  = 389;
+  const DEFAULT_PORT_LDAPS = 636;
+
   private static $all_ldap_opts;
   private static $config_items;
+  private static $profile_clock;
 
 
   public function __construct()
@@ -112,7 +117,7 @@ class AuthLdap extends Auth
     if (!function_exists('ldap_connect'))
     {
       die("<hr><p><b>ERROR: PHP's 'ldap' extension is not installed/enabled. ".
-          "Please check your MRBS and web server configuration.</b></p><hr>\n");
+          "Please check your web server configuration.</b></p><hr>\n");
     }
 
     // Transfer the values from the config variables into a local
@@ -206,7 +211,7 @@ class AuthLdap extends Auth
     $object = array();
     $object['pass'] = $pass;
 
-    return $this->action('validateUserCallback', $user, $object);
+    return ($this->action('validateUserCallback', $user, $object)) ? $user : false;
   }
 
 
@@ -229,8 +234,6 @@ class AuthLdap extends Auth
   private static function validateUserCallback(&$ldap, $base_dn, $dn, $user_search,
                                                $user, &$object)
   {
-    global $ldap_unbind_between_attempts;
-
     self::debug("base_dn '$base_dn' dn '$dn' user '$user'");
 
     $pass = $object['pass'];
@@ -258,10 +261,6 @@ class AuthLdap extends Auth
           if (!self::ldapBind($ldap, $object['config']['ldap_dn_search_dn'], $object['config']['ldap_dn_search_password']))
           {
             self::debug("rebinding failed: " . self::ldapError($ldap));
-            if ($ldap_unbind_between_attempts)
-            {
-              ldap_unbind($ldap);
-            }
             return false;
           }
           self::debug('rebinding successful');
@@ -310,11 +309,6 @@ class AuthLdap extends Auth
       self::debug("bind to '$dn' failed: ". self::ldapError($ldap));
     }
 
-    if ($ldap_unbind_between_attempts)
-    {
-      ldap_unbind($ldap);
-    }
-
     // return failure if no connection is established
     return false;
   }
@@ -322,13 +316,182 @@ class AuthLdap extends Auth
 
   public function getUser($username)
   {
-    $user = new User($username);
+    static $users = array();  // Cache results for performance
 
-    $user->display_name = $this->getDisplayName($username);
-    $user->email = $this->getEmail($username);
-    $user->level = $this->getLevel($username);
+    if (!isset($username) || ($username === ''))
+    {
+      return null;
+    }
 
-    return $user;
+    if (!isset($users[$username]))
+    {
+      // Check to see if this is the current user.  If it is then we
+      // can save ourselves an LDAP query.
+      $mrbs_user = \MRBS\session()->getCurrentUser();
+      if (isset($mrbs_user) && ($mrbs_user->username === $username))
+      {
+        $user = $mrbs_user;
+      }
+      // Otherwise we'll have to query LDAP.
+      else
+      {
+        $object = array();
+
+        $res = $this->action('getUserCallback', $username, $object);
+        if (!$res || !isset($object['user']))
+        {
+          return null;
+        }
+
+        $user = parent::getUser($username);
+        $keys = array('display_name', 'email', 'level');
+
+        foreach ($keys as $key)
+        {
+          if (isset($object['user'][$key]))
+          {
+            $user->$key = $object['user'][$key];
+          }
+        }
+      }
+
+      // Update the cache (we use the static variable as the cache rather than the
+      // database because the database might be out of date).
+      $users[$username] = $user;
+    }
+
+    return $users[$username];
+  }
+
+
+  /* getUserCallback(&$ldap, $base_dn, $dn, $user_search,
+                     $username, &$object)
+   *
+   * &$ldap       - Reference to the LDAP object
+   * $base_dn     - The base DN
+   * $dn          - The user's DN
+   * $user_search - The LDAP filter to find the user
+   * $username    - The user name
+   * &$object     - Reference to the generic object
+   *
+   * Returns:
+   *   false    - Didn't find a user
+   *   true     - Found a user
+   */
+  private static function getUserCallback(&$ldap, $base_dn, $dn, $user_search,
+                                          $user, &$object)
+  {
+    global $ldap_get_user_email, $ldap_debug_attributes;
+
+    self::debug("base_dn '$base_dn' dn '$dn' user_search '$user_search' user '$user'");
+
+    if (!$ldap || !$base_dn || !$dn || !$user_search)
+    {
+      self::debug("invalid parameters, could not call ldap_read, returning false");
+      return false;
+    }
+
+    $attributes = self::getAttributes($object, $ldap_get_user_email, true);
+
+    self::resetProfileClock();
+    // We suppress the errors because it's possible to get a "No such object" error if
+    // the DN doesn't exist - which it won't if (a) we're searching an array of LDAP hosts
+    // or (b) the DN has been deleted since the booking was made.   But check the error
+    // code afterwards and trigger an error if it was any other kind of error.
+    $res = @ldap_read(
+        $ldap,
+        $dn,
+        "(objectclass=*)",
+        array_values($attributes),
+        0,
+        1
+      );
+    $t = self::getProfileClock();
+
+    if ($res === false)
+    {
+      self::debug("ldap_read() failed: " . self::ldapError($ldap));
+      if (self::LDAP_NO_SUCH_OBJECT !== ($errno = ldap_errno($ldap)))
+      {
+        if ($errno === self::LDAP_SUCCESS)
+        {
+          // The errno is reporting success, but that's just the ldap errno.  If ldap_read()
+          // doesn't get as far as interrogating the directory, it will return false,
+          // but ldap_errno() will return success. To get more details try temporarily
+          // removing the error suppression operator ('@').
+          $message = "ldap_read() failed, not due to an LDAP error but probably due " .
+                     "to an initialization error such as 'Array initialization wrong'";
+        }
+        else
+        {
+          $message = ldap_err2str($errno);
+        }
+        trigger_error($message, E_USER_WARNING);
+      }
+      return false;
+    }
+
+    $n_entries = ldap_count_entries($ldap, $res);
+
+    if ($n_entries === false)
+    {
+      self::debug("No entries found - ldap_count_entries() error");
+      return false;
+    }
+
+    self::debug("$n_entries entries found");
+
+    if ($n_entries === 0)
+    {
+      return false;
+    }
+
+    self::debug("ldap_read() succeeded, taking $t seconds");
+
+    if ($ldap_debug_attributes)
+    {
+      // Repeat the read, this time fetching all the attributes and then write
+      // the attributes and their values to the debug log.  Useful for discovering
+      // attribute names.
+      $res2 = @ldap_read($ldap, $dn, "(objectclass=*)", [], 0, 1);
+      $entry = ldap_first_entry($ldap, $res2);
+      $attribute = ldap_first_attribute($ldap, $entry);
+      while ($attribute)
+      {
+        $values = ldap_get_values($ldap, $entry, $attribute);
+        unset($values['count']);  // We don't need this element
+        self::debug("Attribute: \"$attribute\"; Value(s): \"" . implode('", "', $values) . '"');
+        $attribute = ldap_next_attribute($ldap, $entry);
+      }
+    }
+
+    $entry = ldap_first_entry($ldap, $res);
+    $user = self::getResult($ldap, $entry, $attributes);
+
+    if (!isset($user['username']))
+    {
+      $message = 'No username found.  Check the value of $ldap_user_attrib in the MRBS config file.';
+      $message .= " It is currently set to '" . $object['config']['ldap_user_attrib'] . "'.";
+      self::debug($message);
+      return false;
+    }
+
+    if (!isset($user['display_name']))
+    {
+      $user['display_name'] = $user['username'];
+    }
+
+    if (isset($user['groups']))
+    {
+      if (isset($object['config']['ldap_admin_group_dn']))
+      {
+        $user['level'] = in_array($object['config']['ldap_admin_group_dn'], $user['groups']) ? 2 : 1;
+      }
+    }
+
+    self::debug("User '" . $user['username'] . "' found");
+    $object['user'] = $user;
+    return true;
   }
 
 
@@ -396,8 +559,9 @@ class AuthLdap extends Auth
     }
 
     self::debug("searching with base_dn '$base_dn' and filter '$filter'");
-
+    self::resetProfileClock();
     $res = ldap_search($ldap, $base_dn, $filter, $attributes);
+    $t = self::getProfileClock();
 
     if ($res == false)
     {
@@ -405,7 +569,7 @@ class AuthLdap extends Auth
       return false;
     }
 
-    self::debug(ldap_count_entries($ldap, $res) . " entries found");
+    self::debug(ldap_count_entries($ldap, $res) . " entries found in $t seconds");
 
     $entry = ldap_first_entry($ldap, $res);
 
@@ -436,7 +600,15 @@ class AuthLdap extends Auth
         $attribute = ldap_next_attribute($ldap, $entry);
       }
 
-      $object['users'][] = $user;
+      if (isset($user['username']))
+      {
+        if (!isset($user['display_name']))
+        {
+          $user['display_name'] = $user['username'];
+        }
+        $object['users'][] = $user;
+      }
+
       $entry = ldap_next_entry($ldap, $entry);
     }
 
@@ -444,42 +616,91 @@ class AuthLdap extends Auth
   }
 
 
-  protected function getLevel($username)
+  // Returns an array of attributes for use in an LDAP query
+  private static function getAttributes($object, $include_email=true, $include_groups=true)
   {
-    global $ldap_admin_group_dn;
+    $result = array();
 
-    if (!isset($username) || ($username === ''))
+    // Username
+    $result['username'] = \MRBS\utf8_strtolower($object['config']['ldap_user_attrib']);
+
+    // The display name attribute might not have been set in the config file
+    if (isset($object['config']['ldap_name_attrib']))
     {
-      $level = 0;
-    }
-    elseif ($ldap_admin_group_dn)
-    {
-      $object = array();
-      $res = $this->action('checkAdminGroupCallback', $username, $object);
-      $level = ($res) ? 2 : 1;
-    }
-    else
-    {
-      $level = $this->getDefaultLevel($username);
+      $result['display_name'] = \MRBS\utf8_strtolower($object['config']['ldap_name_attrib']);
     }
 
-    return $level;
+    // The email address
+    if ($include_email && isset($object['config']['ldap_email_attrib']))
+    {
+      $result['email'] = \MRBS\utf8_strtolower($object['config']['ldap_email_attrib']);
+    }
+
+    // The group name attribute might not have been set in the config file
+    if ($include_groups && isset($object['config']['ldap_group_member_attrib']))
+    {
+      $result['groups'] = \MRBS\utf8_strtolower($object['config']['ldap_group_member_attrib']);
+    }
+
+    return $result;
   }
 
 
-  // Get the display name of the user from LDAP.  If none, returns the username
-  protected function getDisplayName($username)
+  // Returns an associative array from the result of an LDAP search
+  private static function getResult($ldap, $entry, $attributes)
   {
-    if (!isset($username) || ($username === ''))
+    // Initialise all keys in the user array, in case an attribute isn't present
+    $attributes_keys = array_keys($attributes);
+    $user = array();
+    foreach ($attributes_keys as $key)
     {
-      return $username;
+      switch ($key)
+      {
+        case 'username':
+        case 'display_name':
+        case 'email':
+          $user[$key] = null;
+          break;
+        case 'groups':
+          $user[$key] = array();
+          break;
+        default:
+          throw new \Exception("Unknown key '$key'");
+      }
     }
 
-    $object = array();
+    $attribute = ldap_first_attribute($ldap, $entry);
 
-    $res = $this->action('getNameCallback', $username, $object);
+    // Loop through all the attributes for this user
+    while ($attribute)
+    {
+      $values = ldap_get_values($ldap, $entry, $attribute);
+      $attribute = \MRBS\utf8_strtolower($attribute);  // ready for the comparisons
 
-    return ($res) ? $object['name'] : $username;
+      if ($attribute == $attributes['username'])
+      {
+        $user['username'] = $values[0];
+      }
+      elseif (isset($attributes['display_name']) && ($attribute == $attributes['display_name']))
+      {
+        $user['display_name'] = $values[0];
+      }
+      elseif (isset($attributes['email']) && ($attribute == $attributes['email']))
+      {
+        $user['email'] = $values[0];
+      }
+      elseif (isset($attributes['groups']) && ($attribute == $attributes['groups']))
+      {
+        for ($i=0; $i<$values['count']; $i++)
+        {
+          $user['groups'][] = $values[$i];
+        }
+      }
+
+      $attribute = ldap_next_attribute($ldap, $entry);
+    }
+
+    return $user;
   }
 
 
@@ -496,64 +717,27 @@ class AuthLdap extends Auth
   *               hosts.  Useful, for example, when you want to get a list of all users.
    *
    * Returns:
-   *   false    - The pair are invalid or do not exist
-   *   string   - The validated username
+   *   boolean   - Whether the action was successful
    */
   public function action($callback, $username, &$object, $keep_going=false)
   {
-    foreach (self::$all_ldap_opts['ldap_host'] as $idx => $host)
+    global $ldap_unbind_between_attempts;
+
+    $result = false;
+
+    for ($idx=0; $idx < count(self::$all_ldap_opts['ldap_host']); $idx++)
     {
-      // establish ldap connection
-      if (isset(self::$all_ldap_opts['ldap_port'][$idx]))
-      {
-        $ldap = ldap_connect($host, self::$all_ldap_opts['ldap_port'][$idx]);
-      }
-      else
-      {
-        $ldap = ldap_connect($host);
-      }
+      // Establish LDAP connection
+      $uri = self::getUri($idx);
+      $ldap = ldap_connect($uri);
 
       // Check that connection was established
       if ($ldap)
       {
-        self::debug("got LDAP connection");
+        self::debug("got LDAP connection using $uri");
 
-        if (isset(self::$all_ldap_opts['ldap_deref'][$idx]))
-        {
-          ldap_set_option($ldap, LDAP_OPT_DEREF, self::$all_ldap_opts['ldap_deref'][$idx]);
-        }
-
-        if (isset(self::$all_ldap_opts['ldap_v3'][$idx]) &&
-            self::$all_ldap_opts['ldap_v3'][$idx])
-        {
-          ldap_set_option($ldap, LDAP_OPT_PROTOCOL_VERSION, 3);
-        }
-
-        if (isset(self::$all_ldap_opts['ldap_client_cert'][$idx]) &&
-            self::$all_ldap_opts['ldap_client_cert'][$idx])
-        {
-          // Requires PHP 7.1.0 or later
-          ldap_set_option($ldap, LDAP_OPT_X_TLS_CERTFILE, self::$all_ldap_opts['ldap_client_cert'][$idx]);
-        }
-
-        if (isset(self::$all_ldap_opts['ldap_client_key'][$idx]) &&
-            self::$all_ldap_opts['ldap_client_key'][$idx])
-        {
-          // Requires PHP 7.1.0 or later
-          ldap_set_option($ldap, LDAP_OPT_X_TLS_KEYFILE, self::$all_ldap_opts['ldap_client_key'][$idx]);
-        }
-
-        if (isset(self::$all_ldap_opts['ldap_tls'][$idx]) &&
-            self::$all_ldap_opts['ldap_tls'][$idx])
-        {
-          ldap_start_tls($ldap);
-        }
-
-        if(isset(self::$all_ldap_opts['ldap_disable_referrals'][$idx]) && self::$all_ldap_opts['ldap_disable_referrals'][$idx])
-        {
-          // Required to do a search on Active Directory for Win 2003+
-          ldap_set_option($ldap, LDAP_OPT_REFERRALS, 0);
-        }
+        // Set any applicable LDAP options
+        self::setOptions($ldap, $idx);
 
         if (isset(self::$all_ldap_opts['ldap_dn_search_attrib'][$idx]))
         {
@@ -623,194 +807,41 @@ class AuthLdap extends Auth
           {
             $object['config'][$item] = self::$all_ldap_opts[$item][$idx];
           }
+          else
+          {
+            $object['config'][$item] = null;
+          }
         }
 
         if (empty($dn))
         {
           self::debug("no DN determined, not calling callback");
-          // If we are keeping going we want to be able to search all the LDAP
-          // directories, so we return false if any one of them fails.
-          if ($keep_going)
-          {
-            return false;
-          }
         }
         else
         {
           $res = self::$callback($ldap, self::$all_ldap_opts['ldap_base_dn'][$idx], $dn,
                                  $user_search, $username, $object);
-          if ($res && !$keep_going)
+          if ($res)
           {
-            return $username;
+            $result = true;
           }
         }
 
+        if ($ldap_unbind_between_attempts)
+        {
+          self::debug("unbinding from $uri");
+          ldap_unbind($ldap);
+        }
+
       } // if ($ldap)
 
-      ldap_unbind($ldap);
-    } // foreach
-
-    return ($keep_going) ? true : false;
-  }
-
-
-  /* getNameCallback(&$ldap, $base_dn, $dn, $user_search,
-                     $username, &$object)
-   *
-   * Get the name of a found user
-   *
-   * &$ldap       - Reference to the LDAP object
-   * $base_dn     - The base DN
-   * $dn          - The user's DN
-   * $user_search - The LDAP filter to find the user
-   * $username    - The user name
-   * &$object     - Reference to the generic object
-   *
-   * Returns:
-   *   false    - Didn't find a user
-   *   true     - Found a user
-   */
-  private static function getNameCallback(&$ldap, $base_dn, $dn, $user_search,
-                                          $username, &$object)
-  {
-    $name_attrib = $object['config']['ldap_name_attrib'];
-
-    self::debug("base_dn '$base_dn' dn '$dn' " .
-                "user_search '$user_search' user '$username'");
-
-    if ($ldap && $base_dn && $dn && $user_search)
-    {
-      $res = ldap_read($ldap,
-                       $dn,
-                       "(objectclass=*)",
-                       array(\MRBS\utf8_strtolower($name_attrib)) );
-
-      if (ldap_count_entries($ldap, $res) > 0)
+      if ($result && !$keep_going)
       {
-        self::debug("search successful");
-        $entries = ldap_get_entries($ldap, $res);
-        $object['name'] = $entries[0][\MRBS\utf8_strtolower($name_attrib)][0];
-
-        self::debug("name is '" . $object['name'] . "'");
-
         return true;
       }
-    }
-    return false;
-  }
+    } // for ()
 
-
-  protected function getEmail($username)
-  {
-    global $ldap_get_user_email;
-
-    if (!isset($username) || $username === '')
-    {
-      return '';
-    }
-
-    if ($ldap_get_user_email)
-    {
-      $object = array();
-      $res = $this->action('getEmailCallback', $username, $object);
-      return ($res) ? $object['email'] : '';
-    }
-
-    return $this->getDefaultEmail($username);
-  }
-
-
-  /* getEmailCallback(&$ldap, $base_dn, $dn, $user_search,
-                      $username, &$object)
-   *
-   * Checks if the specified username/password pair are valid
-   *
-   * &$ldap       - Reference to the LDAP object
-   * $base_dn     - The base DN
-   * $dn          - The user's DN
-   * $user_search - The LDAP filter to find the user
-   * $username    - The user name
-   * &$object     - Reference to the generic object
-   *
-   * Returns:
-   *   false    - Didn't find a user
-   *   true     - Found a user
-   */
-  private static function getEmailCallback(&$ldap, $base_dn, $dn, $user_search,
-                                           $user, &$object)
-  {
-    $email_attrib = $object['config']['ldap_email_attrib'];
-
-    self::debug("base_dn '$base_dn' dn '$dn' user_search '$user_search' user '$user'");
-
-    if ($ldap && $base_dn && $dn && $user_search)
-    {
-      $res = ldap_read($ldap,
-                       $dn,
-                       "(objectclass=*)",
-                       array(\MRBS\utf8_strtolower($email_attrib)) );
-
-      if (ldap_count_entries($ldap, $res) > 0)
-      {
-        self::debug("search successful");
-        $entries = ldap_get_entries($ldap, $res);
-        $object['email'] = $entries[0][\MRBS\utf8_strtolower($email_attrib)][0];
-        self::debug("email is '" . $object['email']. "'");
-        return true;
-      }
-    }
-    return false;
-  }
-
-
-  /* checkAdminGroupCallback(&$ldap, $base_dn, $dn, $user_search,
-                             $username, &$object)
-   *
-   * Checks if the specified username is in an admin group
-   *
-   * &$ldap       - Reference to the LDAP object
-   * $base_dn     - The base DN
-   * $dn          - The user's DN
-   * $user_search - The LDAP filter to find the user
-   * $username    - The user name
-   * &$object     - Reference to the generic object
-   *
-   * Returns:
-   *   false    - Not in the admin group
-   *   true     - In the admin group
-   */
-  private static function checkAdminGroupCallback(&$ldap, $base_dn, $dn, $user_search,
-                                                  $username, &$object)
-  {
-    $admin_group_dn = $object['config']['ldap_admin_group_dn'];
-    $group_member_attrib = $object['config']['ldap_group_member_attrib'];
-
-    self::debug("base_dn '$base_dn' dn '$dn' user_search '$user_search' user '$username'");
-
-    if ($ldap && $base_dn && $dn && $user_search)
-    {
-      $res = ldap_read($ldap,
-                       $dn,
-                       "(objectclass=*)",
-                       array(\MRBS\utf8_strtolower($group_member_attrib)) );
-
-      if (ldap_count_entries($ldap, $res) > 0)
-      {
-        self::debug("search successful '$group_member_attrib'");
-        $entries = ldap_get_entries($ldap, $res);
-        foreach ($entries[0][\MRBS\utf8_strtolower($group_member_attrib)] as $group)
-        {
-          if (strcasecmp($group, $admin_group_dn) == 0)
-          {
-            self::debug("admin group successfully found in user object");
-            return true;
-          }
-        }
-        self::debug("admin group not found in user object");
-      }
-    }
-
-    return false;
+    return $result;
   }
 
 
@@ -836,12 +867,87 @@ class AuthLdap extends Auth
   }
 
 
+  // Gets the full LDAP URI
+  private static function getUri($idx)
+  {
+    // First get the port
+    if (isset(self::$all_ldap_opts['ldap_port'][$idx]))
+    {
+      $port = self::$all_ldap_opts['ldap_port'][$idx];
+    }
+    else
+    {
+      $port = self::DEFAULT_PORT_LDAP;
+    }
+
+    // Now get the scheme and host
+    $host = self::$all_ldap_opts['ldap_host'][$idx];
+    $parsed_url = parse_url($host);
+    if (isset($parsed_url['scheme']))
+    {
+      $scheme = $parsed_url['scheme'];
+      $host = $parsed_url['host'];
+    }
+    else
+    {
+      // Make an educated guess at the scheme
+      $scheme = ($port == self::DEFAULT_PORT_LDAPS) ? 'ldaps' : 'ldap';
+    }
+
+    return "$scheme://$host:$port";
+  }
+
+
+  private static function setOptions($ldap, $idx)
+  {
+    if (isset(self::$all_ldap_opts['ldap_deref'][$idx]))
+    {
+      ldap_set_option($ldap, LDAP_OPT_DEREF, self::$all_ldap_opts['ldap_deref'][$idx]);
+    }
+
+    if (isset(self::$all_ldap_opts['ldap_v3'][$idx]) &&
+        self::$all_ldap_opts['ldap_v3'][$idx])
+    {
+      ldap_set_option($ldap, LDAP_OPT_PROTOCOL_VERSION, 3);
+    }
+
+    if (isset(self::$all_ldap_opts['ldap_client_cert'][$idx]) &&
+        self::$all_ldap_opts['ldap_client_cert'][$idx])
+    {
+      // Requires PHP 7.1.0 or later
+      ldap_set_option($ldap, LDAP_OPT_X_TLS_CERTFILE, self::$all_ldap_opts['ldap_client_cert'][$idx]);
+    }
+
+    if (isset(self::$all_ldap_opts['ldap_client_key'][$idx]) &&
+        self::$all_ldap_opts['ldap_client_key'][$idx])
+    {
+      // Requires PHP 7.1.0 or later
+      ldap_set_option($ldap, LDAP_OPT_X_TLS_KEYFILE, self::$all_ldap_opts['ldap_client_key'][$idx]);
+    }
+
+    if (isset(self::$all_ldap_opts['ldap_tls'][$idx]) &&
+        self::$all_ldap_opts['ldap_tls'][$idx])
+    {
+      ldap_start_tls($ldap);
+    }
+
+    if (isset(self::$all_ldap_opts['ldap_disable_referrals'][$idx]) &&
+        self::$all_ldap_opts['ldap_disable_referrals'][$idx])
+    {
+      // Required to do a search on Active Directory for Win 2003+
+      ldap_set_option($ldap, LDAP_OPT_REFERRALS, 0);
+    }
+  }
+
+
   // Adds extra diagnostic information to ldap_error()
   private static function ldapError ($link_identifier)
   {
     $result = ldap_error($link_identifier);
 
-    if (ldap_get_option($link_identifier, LDAP_OPT_DIAGNOSTIC_MESSAGE, $err) &&
+    // LDAP_OPT_DIAGNOSTIC_MESSAGE is not supported by all LDAP libraries
+    if (defined('LDAP_OPT_DIAGNOSTIC_MESSAGE') &&
+        ldap_get_option($link_identifier, LDAP_OPT_DIAGNOSTIC_MESSAGE, $err) &&
         isset($err) && ($err !== ''))
     {
       $result .= " [$err]";
@@ -853,19 +959,50 @@ class AuthLdap extends Auth
 
   /* debug($message)
    *
-   * Output LDAP debugging, if the configuration variable
-   * $ldap_debug is true.
+   * Output LDAP debugging, if either of the configuration variables
+   * $ldap_debug or $ldap_debug_attributes is true.
    *
    */
   private static function debug($message)
+  {
+    global $ldap_debug, $ldap_debug_attributes;
+
+    if ($ldap_debug || $ldap_debug_attributes)
+    {
+      list($called, $caller) = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2);
+      error_log(
+          "[MRBS DEBUG] " .
+          $caller['class'] . $caller['type'] . $caller['function'] . '(' . $called['line'] . ')' .
+          ": $message"
+        );
+    }
+  }
+
+
+  private static function getProfileClock()
   {
     global $ldap_debug;
 
     if ($ldap_debug)
     {
-      $caller = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2)[1];
-      error_log($caller['class'] . $caller['type'] . $caller['function'] . ": $message");
+      return (\MRBS\get_microtime() - self::$profile_clock);
+    }
+    else
+    {
+      return null;
     }
   }
 
+
+  private static function resetProfileClock()
+  {
+    global $ldap_debug;
+
+    if ($ldap_debug)
+    {
+      self::$profile_clock = \MRBS\get_microtime();
+    }
+  }
+
+
 }
