@@ -1034,7 +1034,7 @@ class System
     $result = $locale;
 
     // Strip anything like '.utf-8' off the end
-    if (strpos($result, '.') !== false)
+    if (utf8_strpos($result, '.') !== false)
     {
       $result = strstr($locale, '.', true);
     }
@@ -1228,7 +1228,7 @@ class System
     // Restore the original settings
     foreach ($original_locales as $locale_setting)
     {
-      if (strpos($locale_setting, '=') !== false)
+      if (utf8_strpos($locale_setting, '=') !== false)
       {
         list($category, $locale) = explode('=', $locale_setting);
         // Need to turn the string back into a constant (sometimes PHP doesn't recognise a LC_ constant,
