@@ -98,6 +98,11 @@ $db_tbl_prefix = "mrbs_";
 // MRBS tries to avoid those problems, it is generally better not to use persistent
 // connections if you can.
 $db_persist = false;
+// The number of times to retry getting a database connection in the event that there are
+// too many connections.  [MySQL only at the moment]
+$db_retries = 2;
+// The number of milliseconds to wait before retrying.  [MySQL only at the moment]
+$db_delay = 750; // milliseconds
 
 
 /*********************************
@@ -127,7 +132,8 @@ $mrbs_company = "Your Company";   // This line must always be uncommented ($mrbs
 // Uncomment this next line to use a logo instead of text for your organisation in the header
 //$mrbs_company_logo = "your_logo.gif";    // name of your logo file.   This example assumes it is in the MRBS directory
 
-// Uncomment this next line for supplementary information after your company name or logo
+// Uncomment this next line for supplementary information after your company name or logo.
+// This can contain HTML, for example if you want to include a link.
 //$mrbs_company_more_info = "You can put additional information here";  // e.g. "XYZ Department"
 
 // Uncomment this next line to have a link to your organisation in the header
@@ -289,6 +295,21 @@ $prevent_simultaneous_bookings = false;
 // Set this to true if you want to prevent bookings of a type that is invalid for a room
 $prevent_invalid_types = true;
 
+// The start of the booking day when using periods.  Because MRBS has
+// no notion of when periods actually occur they are assumed to start
+// at the time below when we are enforcing book ahead policies.
+// The setting defines the time of day when bookings open.
+// This should be a in the format hh:mm using the 24 hour clock.
+$periods_booking_opens = '00:00';
+
+// When setting max_create_ahead and max_delete_ahead policies, the time interval is normally
+// measured to the end time of the booking.  This is to prevent users cheating the system by
+// booking a very long slot with the start time just inside the limit and then either not using
+// the early part of the booking, or else editing it down to what they actually need later.
+// However this is not very intuitive for users who might expect the measurement to be relative
+// to the start time, in which case this can be achieved by changing this setting to true.
+$measure_max_to_start_time = false;
+
 /******************
  * Display settings
  ******************/
@@ -620,6 +641,22 @@ $is_mandatory_field['users.display_name'] = true;
 // other people can register.
 $enable_registration = true;
 
+// The default setting for new entries
+$allow_registration_default = false;
+// Whether a limit on the number of registrants is set by default
+$registrant_limit_enabled_default = true;
+// The default maximum number of registrations allowed
+$registrant_limit_default = 1;
+// Whether the registration opens time is enabled by default
+$registration_opens_enabled_default = false;
+// The default time (in seconds) in advance of the start time when registration opens
+$registration_opens_default = 60*60*24*14; // 2 weeks
+// Whether the registration closes time is enabled by default
+$registration_closes_enabled_default = false;
+// The default time (in seconds) in advance of the start time when registration closes
+$registration_closes_default = 0;
+
+
 // Set $skip_default to true if you want the "Skip past conflicts" box
 // on the edit_entry form to be checked by default.  (This will mean that
 // if you make a repeat booking and some of the repeat dates are already
@@ -844,7 +881,9 @@ $auth['db_ext']['password_format'] = 'md5';
 //
 // $ldap_host = array('localhost', 'otherhost.example.com');
 
-// Where is the LDAP server.
+// Where is the LDAP server.   This should ideally consist of a scheme and
+// a host, eg "ldap://foo.com" or "ldaps://bar.com", but just a hostname
+// is supported for backwards compatibility.
 // This can be an array.
 //$ldap_host = "localhost";
 
@@ -869,7 +908,8 @@ $ldap_tls = false;
 // This can be an array.
 //$ldap_base_dn = "ou=organizationalunit,dc=example,dc=com";
 
-// Attribute within the base dn that contains the username
+// Attribute within the base dn that contains the username.
+// In Microsoft AD directories this is "sAMAccountName".
 // This can be an array.
 //$ldap_user_attrib = "uid";
 
@@ -950,6 +990,10 @@ $ldap_suppress_invalid_credentials = true;
 // Output debugging information for LDAP actions
 $ldap_debug = false;
 
+// Output debugging information for LDAP actions and also attribute names and values.
+// A higher level of debugging, useful for discovering attribute names.
+$ldap_debug_attributes = false;
+
 // 'auth_imap' configuration settings
 // See AUTHENTICATION for details of how check against multiple servers
 // Where is the IMAP server
@@ -1070,6 +1114,16 @@ $auth['allow_local_part_email'] = false;
 // set this variable to true
 $auth['only_admin_can_book'] = false;
 
+// This allows you to set a date (and time) before which only admins can make
+// bookings.  This is useful if you want to set a "go live" date and time for MRBS.
+// The variable should be set to a valid date/time format as described in
+// https://www.php.net/manual/en/datetime.formats.php.   For example set
+// $auth['only_admin_can_book_before'] = "2020-12-31 18:00";
+// if you want booking to go live at 6pm on 31 Dec 2020.
+// Note that $auth['only_admin_can_book_before'] will only be considered if
+// $auth['only_admin_can_book'] is false.
+$auth['only_admin_can_book_before'] = false;
+
 // If you want only administrators to be able to make repeat bookings,
 // set this variable to true
 $auth['only_admin_can_book_repeat'] = false;
@@ -1083,6 +1137,10 @@ $auth['only_admin_can_book_multiday'] = false;
 // making separate bookings for the same time slot, but it does slow them down).
 $auth['only_admin_can_select_multiroom'] = false;
 
+// Set this to true if you want to restrict the ability to use the "Copy" button on
+// the view_entry page to ordinary users viewing their own entries and to admins.
+$auth['only_admin_can_copy_others_entries'] = false;
+
 // If you don't want ordinary users to be able to see the other users'
 // details then set this to true.  (Only relevant when using 'db' authentication]
 $auth['only_admin_can_see_other_users'] = false;
@@ -1091,6 +1149,9 @@ $auth['only_admin_can_see_other_users'] = false;
 // not shown unless you have write access to the booking.
 $auth['show_registrant_names'] = false;
 
+// Set this to true if you want ordinary users to be able to register others.
+$auth['users_can_register_others'] = false;
+
 // Set this to true if you don't want admins to be able to make bookings
 // on behalf of other users
 $auth['admin_can_only_book_for_self'] = false;
@@ -1099,9 +1160,13 @@ $auth['admin_can_only_book_for_self'] = false;
 $auth['admin_only_types'] = array();  // eg array('E');
 
 // If you want to prevent the public (ie un-logged in users) from
-// being able to view bookings, set this variable to true
+// being able to view bookings completely, set this variable to true
 $auth['deny_public_access'] = false;
 
+// Or else you can allow them to see that there is a booking, but the
+// details will be shown as private if you set this to true.
+$auth['force_private_for_guests'] = false;
+
 // Set to true if you want admins to be able to perform bulk deletions
 // on the Report page.  (It also only shows up if JavaScript is enabled)
 $auth['show_bulk_delete'] = false;
@@ -1351,6 +1416,10 @@ $faqfilelang = "";
 // Language selection when run from the command line
 $cli_language = "en";
 
+// Set to true to get debug information on languages and locales written to the
+// error log.
+$language_debug = false;
+
 // Vocab overrides
 // ---------------
 
@@ -1430,9 +1499,14 @@ $booking_types[] = "I";
 // still have a type associated with it in the database, which will be the default type.)
 // unset($booking_types);
 
-// Default short description for new bookings
+// Default brief description for new bookings
 $default_name = "";
 
+// Set this to true if you want the booking name (brief description) to
+// default to the current user's display name.  If set, this setting overrides
+// $default_name.
+$default_name_display_name = false;
+
 // Default long description for new bookings
 $default_description = "";
 
