@@ -34,7 +34,10 @@ function generate_registrant_table($row, $previous_page=null)
   {
     echo '<tr>';
     echo '<td>';
-    if (getWritable($registrant['username'], $row['room_id']))
+    // A registration can be cancelled by the registrant or by
+    // the person who registered them or by a booking admin
+    if (getWritable($registrant['username'], $row['room_id']) ||
+        getWritable($registrant['create_by'], $row['room_id']))
     {
       generate_cancel_registration_button(
         $row,
@@ -103,8 +106,8 @@ function get_returl($previous_page=null)
 
 function generate_cancel_registration_button(array $row, array $registrant, $label_text, $previous_page=null, $as_field=false)
 {
-  // Ordinary users cannot cancel registrations after the event has started
-  if (!is_book_admin($row['room_id']) && (time() > $row['start_time']))
+  // Check that it is not too late for ordinary users to cancel
+  if (!is_book_admin($row['room_id']) && entry_registration_cancellation_has_closed($row))
   {
     return;
   }
@@ -154,8 +157,14 @@ function generate_cancel_registration_button(array $row, array $registrant, $lab
 
 function generate_register_button($row, $previous_page=null)
 {
+  // Check that the user is an an admin or else that the entry is open for registration
+  if (!is_book_admin($row['room_id']) && !entry_registration_is_open($row))
+  {
+    return;
+  }
+
   $mrbs_user = session()->getCurrentUser();
-  $can_register_others = is_book_admin($row['room_id']);
+  $can_register_others = can_register_others($row['room_id']);
 
   $form = new Form();
   $form->setAttributes(array('action' => multisite('registration_handler.php'),
@@ -203,15 +212,15 @@ function generate_register_button($row, $previous_page=null)
 
 function generate_event_registration($row, $previous_page=null)
 {
-  global $auth, $server;
+  global $auth;
 
   if (empty($row['allow_registration']))
   {
     return;
   }
 
-  $can_register_others = is_book_admin($row['room_id']);
-  $can_see_others = $auth['show_registrant_names'] || getWritable($row['create_by'], $row['room_id']);
+  $can_register_others = can_register_others($row['room_id']);
+  $can_see_others = $can_register_others || $auth['show_registrant_names'] || getWritable($row['create_by'], $row['room_id']);
   $n_registered = count($row['registrants']);
 
 
@@ -220,7 +229,23 @@ function generate_event_registration($row, $previous_page=null)
   echo "<table class=\"list\">\n";
   echo "<tbody>\n";
 
-  if (!empty($row['enable_registrant_limit']))
+  if ($row['registration_opens_enabled'])
+  {
+    echo '<tr>';
+    echo '<td>' . htmlspecialchars(get_vocab('registration_opens')) . '</td>';
+    echo '<td>' . htmlspecialchars(time_date_string(registration_opens_timestamp($row))) . '</td>';
+    echo "</tr>\n";
+  }
+
+  if ($row['registration_closes_enabled'])
+  {
+    echo '<tr>';
+    echo '<td>' . htmlspecialchars(get_vocab('registration_closes')) . '</td>';
+    echo '<td>' . htmlspecialchars(time_date_string(registration_closes_timestamp($row))) . '</td>';
+    echo "</tr>\n";
+  }
+
+  if (!empty($row['registrant_limit_enabled']))
   {
     echo '<tr>';
     echo '<td>' . htmlspecialchars(get_vocab('registrant_limit')) . '</td>';
@@ -274,7 +299,7 @@ function generate_event_registration($row, $previous_page=null)
   }
   else
   {
-    if (empty($row['enable_registrant_limit']) ||
+    if (empty($row['registrant_limit_enabled']) ||
       ($row['registrant_limit'] > $n_registered))
     {
       generate_register_button($row, $previous_page);
@@ -492,11 +517,14 @@ if (!isset($returl))
   if (isset($server['HTTP_REFERER']) && (substr($server['HTTP_REFERER'], -1) != '/'))
   {
     $parsed_url = parse_url($server['HTTP_REFERER']);
-    $returl = basename($parsed_url['path']);
+    if (isset($parsed_url['path']))
+    {
+      $returl = basename($parsed_url['path']);
+    }
   }
-  // If we haven't got a referer (eg we've come here from an email) then construct
+  // If we still haven't got a referer (eg if we've come here from an email) then construct
   // a sensible place to go to afterwards
-  else
+  if (!isset($returl))
   {
     $returl = 'index.php';
   }
@@ -790,6 +818,15 @@ echo "</table>\n";
 
 echo "<div id=\"view_entry_nav\">\n";
 
+// Set the return URL.  If $previous_page is set then it means that we've come from
+// the registration handler and the original return URL is held in $previous_page.
+// TODO: simplify the code concerning return urls, target urls and the previous page
+// TODO: throughout MRBS.
+if (isset($previous_page))
+{
+  $returl = $previous_page;
+}
+
 // Only show the links for Edit and Delete if the room is enabled.    We're
 // allowed to view and copy existing bookings in disabled rooms, but not to
 // modify or delete them.
@@ -893,33 +930,34 @@ if (!$room_disabled)
 }
 
 // Copy and Copy Series
-echo "<div>\n";
-if (!$series)
+if (!$auth['only_admin_can_copy_others_entries'] || $writeable)
 {
   echo "<div>\n";
-  $params = array('action' => multisite('edit_entry.php'),
-                  'value'  => get_vocab('copyentry'),
-                  'inputs' => array('id' => $id,
-                                    'copy' => 1,
-                                    'returl' => $returl)
-                 );
-  generate_button($params);
+  if (!$series) {
+    echo "<div>\n";
+    $params = array('action' => multisite('edit_entry.php'),
+      'value' => get_vocab('copyentry'),
+      'inputs' => array('id' => $id,
+        'copy' => 1,
+        'returl' => $returl)
+    );
+    generate_button($params);
+    echo "</div>\n";
+  }
+  if ((!empty($repeat_id) || $series) && $repeats_allowed) {
+    echo "<div>\n";
+    $params = array('action' => multisite("edit_entry.php?day=$day&month=$month&year=$year"),
+      'value' => get_vocab('copyseries'),
+      'inputs' => array('id' => $id,
+        'edit_type' => 'series',
+        'copy' => 1,
+        'returl' => $returl)
+    );
+    generate_button($params);
+    echo "</div>\n";
+  }
   echo "</div>\n";
 }
-if ((!empty($repeat_id) || $series) && $repeats_allowed)
-{
-  echo "<div>\n";
-  $params = array('action' => multisite("edit_entry.php?day=$day&month=$month&year=$year"),
-                  'value'  => get_vocab('copyseries'),
-                  'inputs' => array('id' => $id,
-                                    'edit_type' => 'series',
-                                    'copy' => 1,
-                                    'returl' => $returl)
-                 );
-  generate_button($params);
-  echo "</div>\n";
-}
-echo "</div>\n";
 
 // Export and Export Series
 if (!$keep_private && !$enable_periods)
